/* @(#) random.c 96/04/26 1.19 */

#include <kernel/types.h>
#include <kernel/kernel.h>
#include <kernel/random.h>
#include <kernel/super.h>
#include <kernel/sysinfo.h>

/**
|||	AUTODOC -public -class Kernel -group Miscellaneous -name ReadHardwareRandomNumber
|||	Gets a 32-bit random number.
|||
|||	  Synopsis
|||
|||	    uint32 ReadHardwareRandomNumber(void);
|||
|||	  Description
|||
|||	    This function returns a random number generated by the hardware.
|||	    This is useful for providing a seed to the software random-number
|||	    generator.
|||
|||	  Return Value
|||
|||	    Returns an unsigned 32-bit random number. The number can be
|||	    any integer from 0 to (2^32 - 1), inclusive.
|||
|||	  Implementation
|||
|||	    Folio call implemented in Kernel folio V20.
|||
|||	  Associated Files
|||
|||	    <kernel/random.h>, libc.a
|||
|||	  See Also
|||
|||	    srand(), rand(), urand()
|||
**/

uint32 internalReadHardwareRandomNumber(void)
{
vuint32 *dsppNoiseReg;
uint32   lrand, rrand;

    /* This address is derived from the following expression based on
     * constants from dspp_addresses.h in the audio folio.
     *   DSPX_DATA_MEMORY_LOW [ DSPI_NOISE ]
     */
    dsppNoiseReg = ((vuint32 *) 0x61ffc);

    /* Back-to-back reads of DSPPNoise don't come out as looking
     * too random; so we read it a couple extra times and
     * discard the middle values.
     */
    lrand = *dsppNoiseReg;
    *dsppNoiseReg;
    *dsppNoiseReg;
    rrand = *dsppNoiseReg;

    return (lrand << 16) | rrand;
}
