/*
 *	@(#) testmacros.c 95/09/28 1.2
 *	Copyright 1995, The 3DO Company
 *
 *	Tests the CLT macros
 *
 */

#include <kernel/types.h>
#include <graphics/clt/clt.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>


void	CheckMacros(void);
uint32	CheckDefines(void);


main( int argc, char* argv )
{
	uint32 defineCount;

	(void)&argc;
	(void)&argv;

	CheckMacros();
	defineCount = CheckDefines();
	printf("CLT macros appear OK.  %d defines were checked.\n",
		   defineCount);
}


void CheckMacros(void)
{
	assert (PPCBIT(31) == 0x00000001);
	assert (PPCBIT(0) == 0x80000000);
	assert (PPCBIT(8) == 0x00800000);
	assert (PPCMASK(5,31) == 0x00000005);
	assert (PPCMASK(5,30) == 0x0000000A);
	assert (PPCMASK(5,2) == 0xA0000000);
	assert (PPCSHIFT(5) == (31 - 5));
	assert (CLT_Bits(TXTADDRCNTL,INTERFILTER,0x3) == 0x00000180);
	assert (CLT_Mask(TXTADDRCNTL,INTERFILTER) == 0x00000380);
	assert (CLT_Shift(TXTADDRCNTL,INTERFILTER) == 7);
	assert (CLT_Const(TXTADDRCNTL,INTERFILTER,TRILINEAR) == 3);
	assert (CLT_GetBits(TXTADDRCNTL,INTERFILTER,0x87654321) == 0x6);
	assert (CLT_SetConst(TXTADDRCNTL,INTERFILTER,TRILINEAR) == 0x00000180);
}


#define	Check(x)		assert(x); counter++


uint32 CheckDefines(void)
{
	uint32 counter = 0;

	Check(RA_DCNTLDATA == 0x0010);

	/* General Controls */

	Check(RA_DCNTL == 0x0014);
	Check(CLT_Bits(DCNTL,SYNC,0xFFFFFFFF) == 0x00000020);
	Check(CLT_Bits(DCNTL,PSE,0xFFFFFFFF) == 0x00000010);
	Check(CLT_Bits(DCNTL,DINT,0xFFFFFFFF) == 0x00000008);
	Check(CLT_Bits(DCNTL,JR,0xFFFFFFFF) == 0x00000004);
	Check(CLT_Bits(DCNTL,JA,0xFFFFFFFF) == 0x00000002);
	Check(CLT_Bits(DCNTL,TLD,0xFFFFFFFF) == 0x00000001);

	/* Span Walker / Edge Walker Controls */

	Check(RA_ESCNTL == 0x4000);
	Check(CLT_Bits(ESCNTL,PERSPECTIVEOFF,0xFFFFFFFF) == 0x00000004);
	Check(CLT_Bits(ESCNTL,DUSCAN,0xFFFFFFFF) == 0x00000002);
	Check(CLT_Bits(ESCNTL,DSPOFF,0xFFFFFFFF) == 0x00000001);

	/* Texture Mapper Controls */

	Check(RA_TXTLDCNTL == 0x6404);
	Check(CLT_Bits(TXTLDCNTL,COMPRESSED,0xFFFFFFFF) == 0x00000400);
	Check(CLT_Bits(TXTLDCNTL,LOADMODE,0xFFFFFFFF) == 0x00000300);
	Check(CLT_SetConst(TXTLDCNTL,LOADMODE,TEXTURE) == 0x00000000);
	Check(CLT_SetConst(TXTLDCNTL,LOADMODE,MMDMA) == 0x00000100);
	Check(CLT_SetConst(TXTLDCNTL,LOADMODE,PIP) == 0x00000200);
	Check(CLT_Bits(TXTLDCNTL,SRCBITOFF,0xFFFFFFFF) == 0x00000007);

	Check(RA_TXTADDRCNTL == 0x6408);
	Check(CLT_Bits(TXTADDRCNTL,TEXTUREENABLE,0xFFFFFFFF) == 0x00002000);
	Check(CLT_Bits(TXTADDRCNTL,MINFILTER,0xFFFFFFFF) == 0x00001C00);
	Check(CLT_SetConst(TXTADDRCNTL,MINFILTER,POINT) == 0x00000000);
	Check(CLT_SetConst(TXTADDRCNTL,MINFILTER,BILINEAR) == 0x00000800);
	Check(CLT_Bits(TXTADDRCNTL,INTERFILTER,0xFFFFFFFF) == 0x00000380);
	Check(CLT_SetConst(TXTADDRCNTL,INTERFILTER,POINT) == 0x00000000);
	Check(CLT_SetConst(TXTADDRCNTL,INTERFILTER,LINEAR) == 0x00000080);
	Check(CLT_SetConst(TXTADDRCNTL,INTERFILTER,BILINEAR) == 0x00000100);
	Check(CLT_SetConst(TXTADDRCNTL,INTERFILTER,TRILINEAR) == 0x00000180);
	Check(CLT_Bits(TXTADDRCNTL,MAGFILTER,0xFFFFFFFF) == 0x00000070);
	Check(CLT_SetConst(TXTADDRCNTL,MAGFILTER,POINT) == 0x00000000);
	Check(CLT_SetConst(TXTADDRCNTL,MAGFILTER,BILINEAR) == 0x00000020);
	Check(CLT_Bits(TXTADDRCNTL,LODMAX,0xFFFFFFFF) == 0x0F);

	Check(RA_TXTPIPCNTL == 0x640C);
	Check(CLT_Bits(TXTPIPCNTL,PIPSSBSELECT,0xFFFFFFFF) == 0x0001C000);
	Check(CLT_SetConst(TXTPIPCNTL,PIPSSBSELECT,CONSTANT) == 0x00000000);
	Check(CLT_SetConst(TXTPIPCNTL,PIPSSBSELECT,TEXTURE) == 0x00004000);
	Check(CLT_SetConst(TXTPIPCNTL,PIPSSBSELECT,PIP) == 0x00008000);
	Check(CLT_Bits(TXTPIPCNTL,PIPALPHASELECT,0xFFFFFFFF) == 0x00003800);
	Check(CLT_SetConst(TXTPIPCNTL,PIPALPHASELECT,CONSTANT) == 0x00000000);
	Check(CLT_SetConst(TXTPIPCNTL,PIPALPHASELECT,TEXTURE) == 0x00000800);
	Check(CLT_SetConst(TXTPIPCNTL,PIPALPHASELECT,PIP) == 0x00001000);
	Check(CLT_Bits(TXTPIPCNTL,PIPCOLORSELECT,0xFFFFFFFF) == 0x00000700);
	Check(CLT_SetConst(TXTPIPCNTL,PIPCOLORSELECT,CONSTANT) == 0x00000000);
	Check(CLT_SetConst(TXTPIPCNTL,PIPCOLORSELECT,TEXTURE) == 0x00000100);
	Check(CLT_SetConst(TXTPIPCNTL,PIPCOLORSELECT,PIP) == 0x00000200);
	Check(CLT_Bits(TXTPIPCNTL,PIPINDEXOFFSET,0xFFFFFFFF) == 0x000000FF);

	Check(RA_TXTTABCNTL == 0x6410);
	Check(CLT_Bits(TXTTABCNTL,FIRSTCOLOR,0xFFFFFFFF) == 0x00000007);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,PRIMALPHA) == 0x00000000);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,PRIMCOLOR) == 0x00000001);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,TEXALPHA) == 0x00000002);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,TEXCOLOR) == 0x00000003);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,CONSTALPHA) == 0x00000004);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTCOLOR,CONSTCOLOR) == 0x00000005);
	Check(CLT_Bits(TXTTABCNTL,SECONDCOLOR,0xFFFFFFFF) == 0x00000038);
	Check(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,PRIMALPHA) == 0x00000000);
	Check(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,PRIMCOLOR) == 0x00000008);
	Check(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,TEXALPHA) == 0x00000010);
	Check(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,TEXCOLOR) == 0x00000018);
	Check(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,CONSTALPHA) == 0x00000020);
	Check(CLT_SetConst(TXTTABCNTL,SECONDCOLOR,CONSTCOLOR) == 0x00000028);
	Check(CLT_Bits(TXTTABCNTL,THIRDCOLOR,0xFFFFFFFF) == 0x000001C0);
	Check(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,PRIMALPHA) == 0x00000000);
	Check(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,PRIMCOLOR) == 0x00000040);
	Check(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,TEXALPHA) == 0x00000080);
	Check(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,TEXCOLOR) == 0x000000C0);
	Check(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,CONSTALPHA) == 0x00000100);
	Check(CLT_SetConst(TXTTABCNTL,THIRDCOLOR,CONSTCOLOR) == 0x00000140);
	Check(CLT_Bits(TXTTABCNTL,COLOROUT,0xFFFFFFFF) == 0x00000600);
	Check(CLT_SetConst(TXTTABCNTL,COLOROUT,PRIMCOLOR) == 0x00000000);
	Check(CLT_SetConst(TXTTABCNTL,COLOROUT,TEXCOLOR) == 0x00000200);
	Check(CLT_SetConst(TXTTABCNTL,COLOROUT,BLEND) == 0x00000400);
	Check(CLT_Bits(TXTTABCNTL,FIRSTALPHA,0xFFFFFFFF) == 0x00001800);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTALPHA,PRIMALPHA) == 0x00000000);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTALPHA,TEXALPHA) == 0x00000800);
	Check(CLT_SetConst(TXTTABCNTL,FIRSTALPHA,CONSTALPHA) == 0x00001000);
	Check(CLT_Bits(TXTTABCNTL,SECONDALPHA,0xFFFFFFFF) == 0x00006000);
	Check(CLT_SetConst(TXTTABCNTL,SECONDALPHA,PRIMALPHA) == 0x00000000);
	Check(CLT_SetConst(TXTTABCNTL,SECONDALPHA,TEXALPHA) == 0x00002000);
	Check(CLT_SetConst(TXTTABCNTL,SECONDALPHA,CONSTALPHA) == 0x00004000);
	Check(CLT_Bits(TXTTABCNTL,ALPHAOUT,0xFFFFFFFF) == 0x00018000);
	Check(CLT_SetConst(TXTTABCNTL,ALPHAOUT,PRIMALPHA) == 0x00000000);
	Check(CLT_SetConst(TXTTABCNTL,ALPHAOUT,TEXALPHA) == 0x00008000);
	Check(CLT_SetConst(TXTTABCNTL,ALPHAOUT,BLEND) == 0x00010000);
	Check(CLT_Bits(TXTTABCNTL,BLENDOP,0xFFFFFFFF) == 0x00020000);
	Check(CLT_SetConst(TXTTABCNTL,BLENDOP,LERP) == 0x00000000);
	Check(CLT_SetConst(TXTTABCNTL,BLENDOP,MULT) == 0x00020000);

	Check(RA_TXTLODBASE0 == 0x6414);

	Check(RA_TXTLODBASE1 == 0x6418);

	Check(RA_TXTLODBASE2 == 0x641C);
	
	Check(RA_TXTLODBASE3 == 0x6420);

	Check(RA_TXTLDSRCADDR == 0x6424);

	Check(RA_TXTCOUNT == 0x6428);
	Check(CLT_Bits(TXTCOUNT,VALUE,0xFFFFFFFF) == 0x00FFFFFF);

	Check(RA_TXTUVMAX == 0x642C);
	Check(CLT_Bits(TXTUVMAX,UMAX,0xFFFFFFFF) == 0x03FF0000);
	Check(CLT_Bits(TXTUVMAX,VMAX,0xFFFFFFFF) == 0x000003FF);

	Check(RA_TXTUVMASK == 0x6430);
	Check(CLT_Bits(TXTUVMASK,UMASK,0xFFFFFFFF) == 0x03FF0000);
	Check(CLT_Bits(TXTUVMASK,VMASK,0xFFFFFFFF) == 0x000003FF);

	Check(RA_TXTSRCTYPE01 == 0x6434);
	Check(CLT_Bits(TXTSRCTYPE01,CDEPTH0,0xFFFFFFFF) == 0x0000000F);
	Check(CLT_Bits(TXTSRCTYPE01,ADEPTH0,0xFFFFFFFF) == 0x000000F0);
	Check(CLT_Bits(TXTSRCTYPE01,ISTRANS0,0xFFFFFFFF) == 0x00000100);
	Check(CLT_Bits(TXTSRCTYPE01,HASSSB0,0xFFFFFFFF) == 0x00000200);
	Check(CLT_Bits(TXTSRCTYPE01,HASCOLOR0,0xFFFFFFFF) == 0x00000400);
	Check(CLT_Bits(TXTSRCTYPE01,HASALPHA0,0xFFFFFFFF) == 0x00000800);
	Check(CLT_Bits(TXTSRCTYPE01,ISLITERAL0,0xFFFFFFFF) == 0x00001000);
	Check(CLT_Bits(TXTSRCTYPE01,CDEPTH1,0xFFFFFFFF) == 0x000F0000);
	Check(CLT_Bits(TXTSRCTYPE01,ADEPTH1,0xFFFFFFFF) == 0x00F00000);
	Check(CLT_Bits(TXTSRCTYPE01,ISTRANS1,0xFFFFFFFF) == 0x01000000);
	Check(CLT_Bits(TXTSRCTYPE01,HASSSB1,0xFFFFFFFF) == 0x02000000);
	Check(CLT_Bits(TXTSRCTYPE01,HASCOLOR1,0xFFFFFFFF) == 0x04000000);
	Check(CLT_Bits(TXTSRCTYPE01,HASALPHA1,0xFFFFFFFF) == 0x08000000);
	Check(CLT_Bits(TXTSRCTYPE01,ISLITERAL1,0xFFFFFFFF) == 0x10000000);

	Check(RA_TXTSRCTYPE23 == 0x6438);
	Check(CLT_Bits(TXTSRCTYPE23,CDEPTH2,0xFFFFFFFF) == 0x0000000F);
	Check(CLT_Bits(TXTSRCTYPE23,ADEPTH2,0xFFFFFFFF) == 0x000000F0);
	Check(CLT_Bits(TXTSRCTYPE23,ISTRANS2,0xFFFFFFFF) == 0x00000100);
	Check(CLT_Bits(TXTSRCTYPE23,HASSSB2,0xFFFFFFFF) == 0x00000200);
	Check(CLT_Bits(TXTSRCTYPE23,HASCOLOR2,0xFFFFFFFF) == 0x00000400);
	Check(CLT_Bits(TXTSRCTYPE23,HASALPHA2,0xFFFFFFFF) == 0x00000800);
	Check(CLT_Bits(TXTSRCTYPE23,ISLITERAL2,0xFFFFFFFF) == 0x00001000);
	Check(CLT_Bits(TXTSRCTYPE23,CDEPTH3,0xFFFFFFFF) == 0x000F0000);
	Check(CLT_Bits(TXTSRCTYPE23,ADEPTH3,0xFFFFFFFF) == 0x00F00000);
	Check(CLT_Bits(TXTSRCTYPE23,ISTRANS3,0xFFFFFFFF) == 0x01000000);
	Check(CLT_Bits(TXTSRCTYPE23,HASSSB3,0xFFFFFFFF) == 0x02000000);
	Check(CLT_Bits(TXTSRCTYPE23,HASCOLOR3,0xFFFFFFFF) == 0x04000000);
	Check(CLT_Bits(TXTSRCTYPE23,HASALPHA3,0xFFFFFFFF) == 0x08000000);
	Check(CLT_Bits(TXTSRCTYPE23,ISLITERAL3,0xFFFFFFFF) == 0x10000000);

	Check(RA_TXTEXPTYPE == 0x643C);
	Check(CLT_Bits(TXTEXPTYPE,CDEPTH,0xFFFFFFFF) == 0x0000000F);
	Check(CLT_Bits(TXTEXPTYPE,ADEPTH,0xFFFFFFFF) == 0x000000F0);
	Check(CLT_Bits(TXTEXPTYPE,ISTRANS,0xFFFFFFFF) == 0x00000100);
	Check(CLT_Bits(TXTEXPTYPE,HASSSB,0xFFFFFFFF) == 0x00000200);
	Check(CLT_Bits(TXTEXPTYPE,HASCOLOR,0xFFFFFFFF) == 0x00000400);
	Check(CLT_Bits(TXTEXPTYPE,HASALPHA,0xFFFFFFFF) == 0x00000800);
	Check(CLT_Bits(TXTEXPTYPE,ISLITERAL,0xFFFFFFFF) == 0x00001000);

	Check(RA_TXTCONST0 == 0x6440);
	Check(CLT_Bits(TXTCONST0,SSB,0xFFFFFFFF) == 0x80000000);
	Check(CLT_Bits(TXTCONST0,ALPHA,0xFFFFFFFF) == 0x7F000000);
	Check(CLT_Bits(TXTCONST0,RED,0xFFFFFFFF) == 0x00FF0000);
	Check(CLT_Bits(TXTCONST0,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check(CLT_Bits(TXTCONST0,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check(RA_TXTCONST1 == 0x6444);
	Check(CLT_Bits(TXTCONST1,SSB,0xFFFFFFFF) == 0x80000000);
	Check(CLT_Bits(TXTCONST1,ALPHA,0xFFFFFFFF) == 0x7F000000);
	Check(CLT_Bits(TXTCONST1,RED,0xFFFFFFFF) == 0x00FF0000);
	Check(CLT_Bits(TXTCONST1,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check(CLT_Bits(TXTCONST1,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check(RA_TXTCONST2 == 0x6448);
	Check(CLT_Bits(TXTCONST2,SSB,0xFFFFFFFF) == 0x80000000);
	Check(CLT_Bits(TXTCONST2,ALPHA,0xFFFFFFFF) == 0x7F000000);
	Check(CLT_Bits(TXTCONST2,RED,0xFFFFFFFF) == 0x00FF0000);
	Check(CLT_Bits(TXTCONST2,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check(CLT_Bits(TXTCONST2,BLUE,0xFFFFFFFF) == 0x000000FF);

	Check(RA_TXTCONST3 == 0x644C);
	Check(CLT_Bits(TXTCONST3,SSB,0xFFFFFFFF) == 0x80000000);
	Check(CLT_Bits(TXTCONST3,ALPHA,0xFFFFFFFF) == 0x7F000000);
	Check(CLT_Bits(TXTCONST3,RED,0xFFFFFFFF) == 0x00FF0000);
	Check(CLT_Bits(TXTCONST3,GREEN,0xFFFFFFFF) == 0x0000FF00);
	Check(CLT_Bits(TXTCONST3,BLUE,0xFFFFFFFF) == 0x000000FF);

	/* Destination Blend Controls */

	/* ... tbd ... */

	return counter;
}
